const guid = "c12b5a4e69860bc968b47223fd-";

function setContainerPosition(container, parent){
    const OFFSET = 2;

    let parentPosition = parent[0].getBoundingClientRect();
    let bounding = container[0].getBoundingClientRect();

    let top = parentPosition.height + OFFSET;

    if (parentPosition.bottom + bounding.height + OFFSET > (window.innerHeight || document.documentElement.clientHeight))
        top = - (bounding.height + OFFSET);

    container.css({
        top: top
    })
}

function toggleLanguageList(e) {
    const parent = getParentPopupByTarget(e.target);
    const parentName = getPopupNameById(parent[0].id);
    const container = parent.find(`.${guid}lng_list_container`);
    const arrow = parent.find(`.${guid}ToDropArrow`);

    if (!container.is(":visible")) {
        arrow.addClass(`${guid}arrowClosed`).animate('fast');
        container.show();
        analyticsTrackEvent(parentName, 'Show Languages List');
    } else {
        arrow.removeClass(`${guid}arrowClosed`);
        container.hide();
        analyticsTrackEvent(parentName, 'Hide Languages List');
    }
    setContainerPosition(container, parent);
}

function toggleDictionary(e) {
    const parent = getParentPopupByTarget(e.target);
    const parentName = getPopupNameById(parent[0].id);
    const container = parent.find(`.${guid}translate-small-div-bottom`);

    $(`.${guid}vocabulary-icon-container .${guid}red-svg`).toggleClass(`${guid}svg-red-border-none`);

    if (!container.is(":visible")) {
        container.show();
        chrome.storage.local.set({dictionaryEnabled: true});

        analyticsTrackEvent(parentName, 'Show Dictionary');
    } else {
        container.hide();
        chrome.storage.local.set({dictionaryEnabled: false});

        analyticsTrackEvent(parentName, 'Hide Dictionary');
    }
}

function onSpeakClick(e) {
    chrome.storage.local.get(['langDst'], function(config) {
        const parent = getParentPopupByTarget(e.target);
        const parentName = getPopupNameById(parent[0].id);

        speakText(lastTextTranslation, config.langDst, $(e.target).parent());
        analyticsTrackEvent(parentName, 'Click Speak', {'lang_dst': config.langDst});
    })
}

function onLanguageSelect(e) {
    const parent = getParentPopupByTarget(e.target);
    const parentName = getPopupNameById(parent[0].id);

    let dropDownArrow = `<div class="${guid}ToDropArrow"><img src="${chrome.extension.getURL('/frame/Images/arrow.png')}" alt="dropdown arrow"></div>`;

    let code = $(this).data('value');
    let name = $(this).data('name');
    let canVoice = $(this).data('voice') || false;
    let searchVal = $(e.target).closest(`.${guid}lng_list_container`).find(`.${guid}small-lng-search`).val();

    let props = {
        lang_dst: code,
        lang_dst_name: name,
        search_query: searchVal
    };

    analyticsTrackEvent(parentName, 'Select language', props);

    chrome.storage.local.set({
        langSrc: 'auto',
        langSrcName: autoDetectionText,
        langSrcSpeakAvailable: false,
        langDst: code,
        langDstName: name,
        langDstSpeakAvailable: canVoice
    });

    $(`.${guid}init`).attr('data-value', code).attr('data-name', name).attr('data-voice', canVoice);

    toggleSpeakButtons(canVoice);

    const imageUrl = `/frame/Images/flags/${code.split('_').pop().toLocaleLowerCase()}-min.png`;

    let selectedLngImg = `
           <img class="${guid}small-select-img"  src="${chrome.extension.getURL(imageUrl)}" alt="language image">
        `;

    $(`.${guid}init`).html(selectedLngImg + dropDownArrow);

    try {
        //const translateText = $(e.target).closest(`.${guid}lng_list_container`).siblings(`.${guid}translatedText`)[0].innerText
        oneWordTranslate(lastTextOriginal);
    } catch (e) {
        oneWordTranslate($('.c12b5a4e69860bc968b47223fd-translatedText').text())
    }

    toggleLanguageList(e);
    //$(`.${guid}lng_list_container`).hide();
}

function openSettingsTabInBigPopup(){
    analyticsTrackEvent('Medium Popup', 'Click settings');
    hideMediumPopup();
    showBigPopup(lastTextOriginal, lastTextTranslation, 'settings'); // Previously text was directly from the container
}

$(function() {
    $(`.${guid}small-select-to`).on("click", `li`, onLanguageSelect);

    $(`.${guid}init`).click(toggleLanguageList);

    $(`.${guid}speaker-icon-container .${guid}sound_inactive`).click(onSpeakClick);

    $(`.${guid}bookmark-item-container`).click(toggleBookmark);

    $(`#${guid}medium-translator-div .${guid}settings-icon-container`).click(openSettingsTabInBigPopup);

    $(`.${guid}vocabulary-icon-container`).click(toggleDictionary);
});