let PLATFORM_NAME = 'browserExtension';
let InstallRedirectURL = 'https://lingvanex.com';
let UninstallRedirectURL = 'https://lingvanex.com';

//Extension default settings
const defaultConfig = {
    langSrc: 'auto',
    langSrcName: 'Auto',
    langSrcSpeakAvailable: false,
    langDst: 'en_US',
    langDstName: 'English',
    langDstSpeakAvailable: true,
    token: null,
    appLng: null,
    userLogged: false,
    dictionaryEnabled: false,
    allowWordAuto: false,
    allowSentenceAuto: false,
    showTranslateIcon: true,
    showSpeakIcon: true,
    speedOfText: 100,
    voiceGender: 'Male',
    justInstalled: true,
    analyticsEnabled: true,
};

let uuidv = null;
let localNotificationTimer;

//time setUp settings
//(minutes)
const languageListUpdateTime = 2;
let adsRefreshInterval = 5;
let interval;

//(seconds)
const restartFunctionAfterErrorTime = 10;

//message text
const userEmailErrorText = 'Incorrect Email and Password Combination';
const onUserLoginErrorText = 'Error during sign in please try again';

//Api base
const _API_BASE_ = "https://lingvanex.com/v2/api";

//notification on email error during Sign in
let emailOrPswError = {
    type: 'basic',
    title: 'Error during sign in',
    message: `${userEmailErrorText}`,
    priority: 1,
    iconUrl: '../frame/images/logo.png'
};

//notification on email error during Sign in
let loginError = {
    type: 'basic',
    title: 'Error during sign in',
    message: `${onUserLoginErrorText}`,
    priority: 1,
    iconUrl: '../frame/images/logo.png'
};

//get localization text from Api
function getLocales(onLoad) {
    $.ajax({
        "async": true,
        "crossDomain": true,
        "url": "https://bots.nordicwise.com/api/v1/locales?platform=browsers",
        "method": "GET",
        "dataType": "json",
    }).done(function ({locales, ok}) {
        if (ok) {
            browser.storage.local.set({locales: locales});
            if (onLoad)
                onLoad(locales);
        } else {
            //if error try after 10 seconds again
            setTimeout(() => {
                getLocales(onLoad)
            }, restartFunctionAfterErrorTime * 1000)
        }
    }).fail(function (err) {

        console.log(err, 'Error during language list loading');

        //if error try after 10 seconds again
        setTimeout(() => {
            getLocales(onLoad)
        }, restartFunctionAfterErrorTime * 1000)

    });
}

const loadLocalesNames = () => {
    $.getJSON(browser.extension.getURL('/frame/languages.json'), function (languages) {
        let lang_re = /([a-zA-Z]+)(?:(_|-|\w)+)/;
        let lang_short = {}
        Object.keys(languages).map(function (key) {
            let names = {}
            Object.keys(languages[key]).map(function (i) {
                names[i] = languages[key][i].split(/\s?(\(|（)/)[0]
            })

            match = key.match(lang_re);
            if (match) {
                let short_key = match[1]
                lang_short[short_key] = names
            } else {
                lang_short[key] = names
            }
            browser.storage.local.set({'localeNames': lang_short})
        });
    });
};

function addContextMenu(lang) {
    browser.storage.local.get(["locales"], function (config) {
        let title = 'Translate "%s"';
        if (config.locales[lang] && config.locales[lang]['translateSelection']) {
            title = config.locales[lang]['translateSelection']
        }
        browser.contextMenus.create({
            title: title,
            contexts: ["selection"],
            onclick: function (info, tab) {
                showBigPopup(tab.id, info.selectionText, '');
            }
        })
    });
}

function showBigPopup(tab_id, inputText, outputText, tabName = 'translate') {
    browser.tabs.sendMessage(tab_id, {
        msg: "showBigPopup",
        inputText: inputText,
        outputText: outputText,
        tabName: tabName
    });
}

//extension init
function init() {
    loadLocalesNames();

    browser.storage.local.get(['langDst'], function (config) {
        if (!config.langDst) {
            browser.storage.local.set(defaultConfig);
        }
    });

    //get localization text from server
    getLocales(function (locales) {
        browser.storage.local.get(['appLng'], function (config) {
            let appLang = config.appLng;
            if (!appLang) {
                let lang = navigator.language;
                lang = lang.split('-')[0];
                if (navigator.language in locales){
                    appLang = navigator.language;
                } else {
                    appLang = 'en'
                }
                browser.storage.local.set({'appLng': appLang});
            }
            addContextMenu(appLang);
        });
    });

    // get Language list from API
    updateLangList();

    //set timers

    //language update timer
    setInterval(updateLangList, 60 * 60 * (languageListUpdateTime * 1000));

    //bookmark list update timer
    //setInterval(updateUserBookmarkList, 60 * 60 * (languageListUpdateTime * 1000));

    //run function for sync user bookmarks on first load
    //updateUserBookmarkList();

    //detect browser and get backenster parameters
    try {
        browser.storage.local.get(['lingvanex_uuid'], function (config) {
            if (!config.lingvanex_uuid) {
                config.lingvanex_uuid = uuidv4();

                browser.storage.local.set({'lingvanex_uuid': config.lingvanex_uuid});
            }
            uuidv = config.lingvanex_uuid;
            console.log("lingvanex_uuid: " + uuidv);

            if (localNotificationTimer)
                clearInterval(localNotificationTimer);

            readBackensterParams(function () {
                getAds();
                if (interval) {
                    localNotificationTimer = setInterval(function () {
                        showNotification();
                    }, interval);
                }
            });

            loadPiwikVars();
        });
    } catch (ex) {
        console.log("lingvanex request error: " + ex);
    }
}

function getAds() {
    debug_log('getAds');
    //timer to update ads
    setTimeout(getAds, 60 * 60 * (adsRefreshInterval * 1000));

    browser.storage.local.get(['appLng'], function ({appLng}) {
        $.ajax({
            "async": true,
            "crossDomain": true,
            "dataType": "json",
            "url": `https://backenster.com/api/advertisementTable/getAd?appKey=${window.LingvanexEnv.backensterKey}&language=${appLng}&uuid=${uuidv}`,
            "method": "GET",
            "headers": {
                "Content-Type": "application/json"
            },
            "processData": false
        }).done(function ({allAdvertisement}) {
            browser.storage.local.set({addInfo: allAdvertisement});
        }).fail((e) => {
            console.log(e.responseText)
        });
    });
}

function readBackensterRedirectUrls(callback) {
    debug_log("readBackensterRedirectUrls");

    $.ajax({
        type: 'post',
        timeout: 15000,
        url: 'https://backenster.com/api/app/config',
        cache: false,
        dataType: 'json',
        data: {
            appKey: window.LingvanexEnv.backensterKey,
            uuid: uuidv
        },
        success: function ({config, err}) {
            if (err) {
                console.log("backenster config error: " + response.err);
            } else if (config) {
                InstallRedirectURL = config.InstallRedirectURL ? config.InstallRedirectURL : InstallRedirectURL;
                UninstallRedirectURL = config.UninstallRedirectURL ? config.UninstallRedirectURL : UninstallRedirectURL;
            }
            if (callback)
                callback();
        },
        error: function (err) {
            console.log("backenster config error: " + err);
            if (callback)
                callback();
        }
    });
}

//backester params read
function readBackensterParams(callback) {
    debug_log('readBackensterParams');
    try {
        let xhr = new XMLHttpRequest();

        let json = JSON.stringify({
            appKey: window.LingvanexEnv.backensterKey,
            uuid: uuidv
        });

        xhr.open("POST", 'https://backenster.com/api/app/config', true);
        xhr.setRequestHeader('Content-type', 'application/json; charset=utf-8');

        xhr.onreadystatechange = function () {
            if (xhr.readyState === 4 && this.status === 200) {
                let response = JSON.parse(this.responseText);
                if (response.err) {
                    console.error(response.err);
                    if (callback)
                        callback();
                    return;
                } else {
                    if (response.config && response.config.ShowLocalNotification
                        && response.config.LocalNotificationFireTime && response.config.LocalNotificationFireTime > 0
                        && response.localNotification && response.localNotification.length > 0) {

                        notifications = response.localNotification;
                        interval = response.config.LocalNotificationFireTime * 1000;
                    }
                    if (callback)
                        callback();
                }
            }
        };
        xhr.send(json);
    } catch (ex) {
        console.log("lingvanex request error: " + ex);
    }
}


//update language list in localStorage
function updateLangList() {
    debug_log('updateLangList');
    $.ajax({
        type: 'get',
        dataType: 'json',
        timeout: 10000,
        headers: {
            'Authorization': 'Fujiwaranosai'
        },
        url: `https://backenster.com/v2/api/v3/getLanguages?platform=${PLATFORM_NAME}&uuid=${uuidv}`,
        success: function (response) {
            if (response.err) {

                setTimeout(updateLangList, restartFunctionAfterErrorTime * 1000);
                console.log('error during lang updating', response.err)
            } else {
                browser.storage.local.set({languageList: response.result});
            }
        },
        error: function (e) {
            setTimeout(updateLangList, 10000);
            console.error(e.statusText);
        }
    });
}

//show browser notification
const showNotification = () => {
    debug_log('showNotification');
    if (localNotificationTimer)
        clearInterval(localNotificationTimer);
    try {
        if (notifications && notifications.length > 0) {
            browser.storage.local.get(['last_notification_id'], function ({last_notification_id}) {
                try {
                    let lastId = last_notification_id || -1;
                    let notification;
                    let filtered = notifications.filter(n => n.id > lastId);
                    if (filtered.length > 0)
                        notification = filtered[0];
                    else
                        return;

                    browser.storage.local.set({'last_notification_id': notification.id});
                    browser.notifications.create(notification.id.toString(), {
                        "type": "basic",
                        "iconUrl": browser.extension.getURL("img/icon48.png"),
                        "title": notification.title,
                        "message": notification.description
                    }, function (notificationId) {
                        console.log("notification id: " + notificationId);
                    });
                } catch (e) {
                    console.error(e);
                }
            });
        }
    } catch (e) {
        console.error(e);
    }
};


//TODO incomprehensible function
function uuidv4() {
    return ([1e7] + -1e3 + -4e3 + -8e3 + -1e11).replace(/[018]/g, c =>
        (c ^ crypto.getRandomValues(new Uint8Array(1))[0] & 15 >> c / 4).toString(16)
    )
}


//tab update event
function handleTabUpdated(tabId, changeInfo, tabInfo) {
    if (tabId == lastOpenedTabId) {
        console.log("Tab Updated: " + lastOpenedTabId);
        lastOpenedTabId = -1;
    }
}


//add sync user bookmark
const addBookmarkToUserSynced = (from, to, originalText, translatedText, token, sendResponse) => {
    if (debug)
        console.log('add to bookmarks: ' + originalText);
    let currentTime = Math.floor(new Date().getTime() / 1000.0);
    let settings = {
        "async": false,
        "crossDomain": true,
        "url": "https://lingvanex.com/v2/api/user/favorite",
        "method": "POST",
        "headers": {
            "Authorization": `Bearer ${token}`,
        },
        "dataType": "json",
        "data": {
            "time": ` ${currentTime} `,
            "textFrom": `${originalText}`,
            "textTo": `${translatedText}`,
            "languageFrom": `${from}`,
            "languageTo": `${to}`,
            "uuid": `${uuidv}`,
            "platform": `${PLATFORM_NAME}`
        }
    };

    $.ajax(settings).done(function (response) {
        if (sendResponse) {
            sendResponse(response);
        }
        if (debug)
            console.log(response, "bookmark added");
    });
};

const deleteSyncBookmark = (id, token, sendResponse) => {
    if (debug)
        console.log('delete bookmark: ' + id);
    let settings = {
        "async": false,
        "crossDomain": true,
        "dataType": "json",
        "url": `https://lingvanex.com/v2/api/user/favorite/${id}`,
        "method": "DELETE",
        "headers": {
            "Authorization": `Bearer ${token}`
        }
    };
    $.ajax(settings).done(function (response) {
        if (sendResponse) {
            sendResponse({data: response});
        }
        if (debug)
            console.log(response, "bookmark deleted: " + id);
    });
};

//get user bookmark list
const getBookmarkList = (token, syncFinished) => { // FIXME: Rename to SyncBookmarks
    browser.storage.local.get(['languageList'], function ({languageList}) {
        let shortLngList = [];
        languageList.map((item) => {
            shortLngList.push({
                "full_code": item.full_code,
                "englishName": item.englishName,
            })
        });

        let settings = {
            "async": true,
            "crossDomain": true,
            "url": `https://lingvanex.com/v2/api/user/favorites?limit=9999&platform=${PLATFORM_NAME}&uuid=${uuidv}&time=0 `,
            "method": "GET",
            "dataType": "json",
            "headers": {
                "Authorization": `Bearer ${token}`,
            }
        };

        $.ajax(settings).done(function ({data}) {
            const {favorites} = data;

            let bookmarkList = favorites.map((bookmarkItem) => {
                const {id, languageFrom, languageTo, textFrom, textTo, time} = bookmarkItem;

                let fromNameItems = shortLngList.filter((lng) => lng.full_code === languageFrom);
                let toNameItems = shortLngList.filter((lng) => lng.full_code === languageTo);

                let fromName = languageFrom;
                let toName = languageTo;

                if (fromNameItems.length > 0) {
                    fromName = fromNameItems[0].englishName
                }

                if (toNameItems.length > 0) {
                    toName = toNameItems[0].englishName
                }


                return {
                    "bookmark": true,
                    "from": `${languageFrom}`,
                    "fromName": `${fromName}`,
                    "id": `${id}`,
                    "originalText": `${textFrom}`,
                    "synced": true,
                    "to": `${languageTo}`,
                    "toName": `${toName}`,
                    "translatedText": `${textTo}`,
                    "time": time
                };
            });
            browser.storage.local.get(['translateHistory'], function ({translateHistory}) {
                if (translateHistory === undefined)
                    translateHistory = [];

                translateHistory.forEach((item) => {
                    item.bookmark = false;
                    item.synced = true;
                });

                bookmarkList.forEach(function (bookmarkItem) {
                    let existingItem = translateHistory.find((item) => {
                        return (item.from === bookmarkItem.from &&
                            item.to === bookmarkItem.to &&
                            item.originalText === bookmarkItem.originalText)
                    });
                    if (existingItem === undefined) {
                        if (debug)
                            console.log(`Added new bookmark: ${bookmarkItem.id} "${bookmarkItem.originalText}"`);
                        translateHistory.push(bookmarkItem)
                    } else {
                        existingItem.id = bookmarkItem.id; // Sync Ids
                        existingItem.bookmark = true;
                        existingItem.synced = true;
                    }
                });
                browser.storage.local.set({translateHistory: translateHistory}, () => {
                    if (syncFinished)
                        syncFinished()
                });
            });
        });
    });
};

function onUserReg(email, password, username, callback) {
    let settings = {
        "async": true,
        "crossDomain": true,
        "url": `${_API_BASE_}/auth/registration`,
        "method": "POST",
        "headers": {
            "Authorization": "Basic am9zaHVhTW9iaWxlQ2xpZW50OkJhX0ByMUY7MzR4UXA",
            "Content-Type": "application/json",
            "Accept": "*/*",
            "Cache-Control": "no-cache",
        },
        "processData": false,
        "dataType": "json",
        "data": `{
                "email": "${email}",
                "password": "${password}",
                "username": "${username}",
                "platform": "${PLATFORM_NAME}",
                "uuid": "${uuidv}",
                "role": "preCustomer"
            }`
    };

    try {
        $.ajax(settings).always(function (response) {
            const statusCode = response.status ? response.status : response.statusCode;
            const data = response.data;

            if (statusCode >= 400 && statusCode <= 500) {
                let notification;
                if (statusCode === 400) {
                    notification = {
                        type: 'basic',
                        title: 'Error during registration',
                        message: `${response.responseJSON.errorCode}`,
                        priority: 1,
                        iconUrl: '../frame/images/logo.png'
                    };
                } else {
                    notification = {
                        type: 'basic',
                        title: 'Error during sign in',
                        message: "error during registration please check data and try again",
                        priority: 1,
                        iconUrl: '../frame/images/logo.png'
                    };
                }
                browser.notifications.create(null, notification);
            } else if (statusCode >= 200 && statusCode <= 300) {
                if (data.bearerToken && data.user.username) {

                    let credentials = {
                        userLogged: true,
                        userName: data.user.username,
                        token: data.bearerToken,
                        id: data.id,
                        email: data.user.email
                    };
                    browser.storage.local.set(credentials);

                    analyticsTrackEvent('Register', 'Success', {
                        userName: credentials.userName,
                        email: credentials.email, user_id: credentials.id
                    });

                    return callback(credentials);
                }
            }
            analyticsTrackEvent('Register', 'Failed');
            return callback();
        });
    } catch (e) {
        analyticsTrackEvent('Register', 'Exception');
        return callback();
    }
}

const onUserLogin = (email, password, callback) => {
    const settings = {
        "async": true,
        "crossDomain": true,
        "url": `${_API_BASE_}/auth/login`,
        "method": "POST",
        "dataType": "json",
        "headers": {
            "Authorization": "Basic am9zaHVhTW9iaWxlQ2xpZW50OkJhX0ByMUY7MzR4UXA",
            "Content-Type": "application/json",
            "Accept": "*/*",
            "Cache-Control": "no-cache",
            "cache-control": "no-cache"
        },
        "processData": false,
        "data": `{
                "email": "${email}",
                "password": "${password}",
                "uuid": "${uuidv}"
            }`
    };

    try {
        $.ajax(settings).always(function (response) {
            const status = response.status ? response.status : response.statusCode;
            const data = response.data;

            if (status >= 400 && status <= 500) {
                if (status === 403) {
                    emailOrPswError.priority = emailOrPswError.priority++;
                    browser.notifications.create(null, emailOrPswError);
                } else {
                    browser.notifications.create(null, loginError);
                }
            } else if (status >= 200 && status <= 300) {
                if (data.bearerToken && data.user.username) {
                    let credentials = {
                        userLogged: true,
                        userName: data.user.username,
                        token: data.bearerToken,
                        id: data.id,
                        email: data.user.email
                    };
                    browser.storage.local.set(credentials);

                    analyticsTrackEvent('Login', 'Success', {
                        userName: credentials.userName,
                        email: credentials.email, user_id: credentials.id
                    });
                    return callback(credentials);
                }
            }
            analyticsTrackEvent('Login', 'Failed');
            return callback();
        });
    } catch (e) {
        analyticsTrackEvent('Login', 'Exception');
        callback();
    }
};

function syncHistory(token, syncFinished) {
    browser.storage.local.get(['translateHistory'], function ({translateHistory}) {
        if (translateHistory === undefined) {
            syncFinished()
        }
        translateHistory.forEach(function (item, index, object) {
            if (item.synced === false) {
                if (item.bookmark === true && item.remove !== true) {
                    addBookmarkToUserSynced(item.from, item.to, item.originalText, item.translatedText, token, (data) => {
                        item.id = data.data.id;
                        item.synced = true;
                    });
                } else {
                    deleteSyncBookmark(item.id, token, () => {
                        item.synced = true;
                    });
                    if (item.remove === true)
                        object.splice(index, 1);
                }
            }
        });

        translateHistory = translateHistory.filter((item) => {
            return item.remove !== true
        });
        browser.storage.local.set({translateHistory: translateHistory}, () => {
            if (syncFinished)
                syncFinished()
        });
    })
}


readBackensterRedirectUrls(function () {
    if (debug)
        console.log('UninstallRedirectURL set to ' + UninstallRedirectURL);
    if (debug === false)
        browser.runtime.setUninstallURL(UninstallRedirectURL);
});

browser.runtime.onInstalled.addListener(function () {
    analyticsTrackEvent('Extension', 'Installed');

    readBackensterRedirectUrls(function () {
        if (debug)
            console.log('InstallRedirectURL is ' + InstallRedirectURL);

        browser.storage.local.set({justInstalled: true, analyticsEnabled: true});

        if (debug === false)
            browser.tabs.create({
                url: InstallRedirectURL,
                active: true
            });
    });
    return false;
});

//On message events
browser.runtime.onMessage.addListener(function (request, sender, sendResponse) {
    let tab = sender.tab;

    if (request && request.lingvanexClosed) {
        if (debug)
            console.log("lingvanexClosed");
        if (localNotificationTimer)
            clearInterval(localNotificationTimer);
        readBackensterParams(function () {
            if (interval) {
                localNotificationTimer = setInterval(function () {
                    showNotification();
                }, interval);
            }
        });
    } else if (request.msg === 'popup-loaded') {
        if (debug)
            console.log('popup-loaded');

        browser.storage.local.get(['token', 'userLogged'], function ({token, userLogged}) {
            if (userLogged) {
                syncHistory(token, () => {
                    getBookmarkList(token, () => {
                        sendResponse();
                    })
                });
            }
        });

        browser.tabs.query({active: true, currentWindow: true}, function (tabs) {
            browser.tabs.sendMessage(tabs[0].id, {msg: "close_translate_popups"});
        });
    } else if (request.msg === "image-parse") {
        dataURItoBlob(request.url).then(image => _imageToOCr(image));
    } else if (request.msg === "load_popup_ended") {
        browser.tabs.query({active: true, currentWindow: true}, function (tabs) {
            browser.tabs.sendMessage(tabs[0].id, {msg: "popupLoaded"});
        });
    } else if (request.msg === "start_bg_image_capturing") {
        browser.tabs.query({active: true, currentWindow: true}, function (tabs) {
            browser.tabs.sendMessage(tabs[0].id, {msg: "start_image_ocr"});
        });
    } else if (request.evt === 'capture-screen') {
        browser.tabs.captureVisibleTab(function (dataURL) {
            browser.tabs.getZoom(tab.id, function (zf) {
                sendResponse({
                    dataURL: dataURL,
                    zf: zf
                });
            });
        });
        return true;
    } else if (request.evt === 'capture-done') {
        sendResponse({
            farewell: 'capture-done:OK'
        });
    } else if (request.msg === 'addSyncBookmarks') {
        // const {originalText, translatedText, from, to, token} = request;
        //
        // addBookmarkToUserSynced(from, to, originalText, translatedText, token, sendResponse);
        //
        //
        // browserLocal.get(['languageList'], function ({languageList}) {
        //     getBookmarkList(token, languageList)
        // });
        //
        // return true
    } else if (request.msg === 'syncBookmarks') {
        browser.storage.local.get(['token', 'userLogged'], function ({token, userLogged}) {
            if (userLogged) {
                syncHistory(token, () => {
                    getBookmarkList(token, () => {
                        sendResponse();
                    })
                });
            }
        });
    } else if (request.msg === 'getCurrentPageUrl') {
        browser.tabs.query({active: true, currentWindow: true}, function (tabs) {
            sendResponse({pageUrl: tabs[0].url});
        });
        return true
    } else if (request.msg === 'userReg') {
        const {email, password, username} = request;
        onUserReg(email, password, username, sendResponse);
        return true
    } else if (request.msg === 'userLogin') {
        const {email, password} = request;
        onUserLogin(email, password, sendResponse);
        return true
    } else if (request.msg === 'updateLanguageList') {
        updateLangList()
    } else if (request.msg === 'tabs-reset') {
        browser.tabs.query({active: true, currentWindow: true}, function (tabs) {
            browser.tabs.sendMessage(tabs[0].id, {msg: "tabs-reset-start"});
        });
        return true
    } else if (request && request.lingvanexOpened) {

        if (localNotificationTimer)
            clearInterval(localNotificationTimer);

        browser.tabs.query({currentWindow: true, active: true}, function (tabs) {
            lastOpenedTabId = tabs[0].id;
            if (debug)
                console.log("Active tab: " + lastOpenedTabId);
        });

        browser.tabs.onUpdated.addListener(handleTabUpdated);
    }
});

async function dataURItoBlob(dataURI) {
    let byteString;
    if (dataURI.split(',')[0].indexOf('base64') >= 0) {
        byteString = atob(dataURI.split(',')[1]);
    } else {
        byteString = unescape(dataURI.split(',')[1]);

    }

    // separate out the mime component
    let mimeString = dataURI.split(',')[0].split(':')[1].split(';')[0];

    // write the bytes of the string to a typed array
    let ia = new Uint8Array(byteString.length);
    for (let i = 0; i < byteString.length; i++) {
        ia[i] = byteString.charCodeAt(i);
    }


    return new Promise((resolve, reject) => {
        resolve(
            new Blob([ia], {type: mimeString})
        )
    });
}


const _imageToOCr = (blob) => {
    browser.tabs.query({active: true, currentWindow: true}, function (tabs) {
        browser.tabs.sendMessage(tabs[0].id, {msg: "start_load_animation"});
    });
    browser.storage.local.get(['langSrc', 'langDst'], function (config) {
        let form = new FormData();
        form.append("from", config.langSrc);
        form.append("to", config.langDst);
        form.append('file', blob, "filename.png");

        let settings = {
            "async": true,
            "crossDomain": true,
            "url": "https://backenster.com/v2/api/v3/parseImage?platform=android&compose=true",
            "method": "POST",
            "headers": {
                "authorization": "Bearer sdf2fsd34lkkdfg"
            },
            "processData": false,
            "contentType": false,
            "dataType": "json",
            "data": form
        };

        $.ajax(settings).done(function (response) {
            const {err, sourceData, translatedData} = response;
            if (err === null) {

                let originalText = '';
                let translatedText = '';

                if (sourceData) {
                    sourceData.map((text) => originalText += ` ${text}`)
                }

                if (translatedData) {
                    translatedData.map((text) => translatedText += ` ${text}`)
                }

                browser.tabs.query({active: true, currentWindow: true}, function (tabs) {
                    browser.tabs.sendMessage(tabs[0].id, {
                        msg: "image_ocr_success",
                        sourceData: originalText,
                        translatedData: translatedText
                    });
                });
            }
        }).fail(({responseJSON}) => {
            if (responseJSON) {
                const {err} = responseJSON;

                browser.tabs.query({active: true, currentWindow: true}, function (tabs) {
                    browser.tabs.sendMessage(tabs[0].id, {msg: "image_ocr_error", error: err});
                });
            } else {
                browser.tabs.query({active: true, currentWindow: true}, function (tabs) {
                    browser.tabs.sendMessage(tabs[0].id, {
                        msg: "image_ocr_error",
                        error: "error during parsing image,please try again"
                    });
                });
            }
        });
    })
};

init();
