﻿let selectedText = '';
var browser = window.chrome || window.browser;
const mediumTranslatorLimit = 30;
const guidContent = "c12b5a4e69860bc968b47223fd-";

const popupX = -2;
const popupY = 20;

$(document).ready(function () {
    init();
});

function init() {
    $(`#lingvanex-container`).hide();

    let translatorIcon = $("#translator-icon");
    let speakIcon = $("#speak-icon");

    translatorIcon.on('mouseup', onTranslationIconClick); // do not loose focus!

    $(`#speak-icon .${guidContent}sound_inactive`).click(onSpeakIconClick1);

    makePopupDraggable()
}


function isInside(event, selector){
    return $(event.target).closest(selector).length > 0
}

function onDocumentClick(e) {
    let reset = false;

    if ($(`#${guidContent}small-translator-div`).is(":visible")) {
        if (!isInside(e, `#${guidContent}small-translator-div`)) {
            hideSmallPopup();
            reset = true;
        }
    }

    if ($(`#${guidContent}medium-translator-div`).is(":visible")) {
        if (!isInside(e, `#${guidContent}medium-translator-div`)) {
            hideMediumPopup();
            reset = true;
        }
    }

    if ($(`#${guidContent}translator-div-container`).is(":visible")) {
        // translator-icon will trigger appearance of big translator window
        if (!isInside(e, `#translator-icon,#${guidContent}translator-div-container`)) {
            hideBigPopup();
            reset = true;
        }
    }

    if (reset){
        clearPopups();
        // updateExtensionData();
    }
}

document.addEventListener('mouseup', e => {
    // if ($(`#${guidContent}translator-div-container`).is(':visible')) {
    //     return
    // }

    const buttonsContainer = $(`#lingvanex-container`);

    if (buttonsContainer.is(":visible")) {
        if (e.target && $(e.target.parentElement).attr("id") === 'speak-icon')
            return;

        hideSquareButtons();
    }

    if ($(`#${guidContent}translator-div,#${guidContent}translator-div-container`).css("display") === "block") {
        browser.runtime.sendMessage({"lingvanexClosed": true});
    }

    onDocumentClick(e);

    if (isInside(e, `#${guidContent}small-translator-div`) ||
        isInside(e,`#${guidContent}medium-translator-div`) ||
        isInside(e, `#translator-icon,#${guidContent}translator-div-container`)
    ) {
        return;
    }

    selectedText = getSelectedText();

    //check selected word length
    if (!translatedTextIsValid(e.target, selectedText)) {
        return;
    }

    browser.storage.local.get(['allowWordAuto', 'allowSentenceAuto', 'showTranslateIcon', 'showSpeakIcon'], function ({allowWordAuto, allowSentenceAuto, showTranslateIcon, showSpeakIcon}) {
        let can_show_icon = true;

        if (isOneWord(selectedText)) {
            if (allowWordAuto === true) {
                can_show_icon = false;
                updateExtensionData();
                showSmallPopup(e.pageX, e.pageY, selectedText)
            }
        } else if (isMediumSentence(selectedText)) {
            if (allowSentenceAuto === true) {
                can_show_icon = false;
                updateExtensionData();
                //clearPopups();
                showMediumPopup(e.pageX, e.pageY, selectedText)
            }
        }
        if (can_show_icon){
            if (showTranslateIcon === true || showSpeakIcon === true) {
                showSquareButtons(e.pageX, e.pageY, selectedText, showTranslateIcon, showSpeakIcon);
            }
        }
    });
});


function clearPopups() {
    //clear text fields in big popup
    $(`#${guidContent}inputText`).val('');
    $(`#${guidContent}outputText`).val('');

    //clear result in small & medium popup
    $(`.${guidContent}translatedText`).text('');

    //stop all audios
    stopAudioPlay($(`.${guidContent}sound_activate`).parent())
}


function hideMediumPopup() {
    if (debug)
        console.debug('hideMediumPopup');

    const popup = $(`#${guidContent}medium-translator-div`);
    if (popup.is(":visible")) {
        //Hide medium popup
        popup.find(`.${guid}lng_list_container`).hide();
        popup.hide();

        //remove active tab icon
        $(`.${guidContent}bookmark-icon`).removeClass(`${guidContent}svg-red`);

        clearPopups();
        analyticsTrackEvent('Medium Popup', 'Hide')
    }
}

const hideBigPopup = () => {
    if (debug)
        console.debug('hideBigPopup');

    const popup = $(`#${guidContent}translator-div-container`);
    if (popup.is(":visible")) {
        //hide big modal container
        popup.hide();

        clearPopups();
        analyticsTrackEvent('Big Popup', 'Hide')
    }
};


const hideSmallPopup = () => {
    if (debug)
        console.debug('hideSmallPopup');

    const popup = $(`#${guidContent}small-translator-div`);
    if (popup.is(":visible")) {
        popup.find(`.${guid}lng_list_container`).hide();
        popup.hide();

        $(`.${guidContent}bookmark-icon`).removeClass(`${guidContent}svg-red`);

        $(`.${guidContent}vocabulary-icon-container svg`).removeClass(`${guidContent}svg-red-border-none`);

        clearPopups();
        analyticsTrackEvent('Small Popup', 'Hide')
    }
};


const updateExtensionData = () => {
    updateSelectedLanguages();
    updateSmallSelectedLanguages();
    checkUserLogin();
};

const clearSelectedText = () => {
    // if (window.getSelection) {
    //     if (window.getSelection().empty) {  // browser
    //         window.getSelection().empty();
    //     } else if (window.getSelection().removeAllRanges) {  // Firefox
    //         window.getSelection().removeAllRanges();
    //     }
    // }
};

const translatedTextIsValid = (target, selectedText) => {
    if ($(target).closest(`#${guidContent}medium-translator-div,.swal-modal`).length ||
        $(target).closest(`#${guidContent}small-translator-div`).length)
        return false;

    if (selectedText === null || selectedText.trim().length === 0)
        return false;
    return true
};

function showSquareButtons(left, top, selectedText, showTranslateIcon, showSpeakIcon){
    const OFFSET = 15;
    const buttonsContainer = $(`#lingvanex-container`);
    const translateIcon = $(`#translator-icon`);
    const speakIconContainer = $(`#speak-icon`);

    let x = 0;
    if (isOneWord(selectedText)) {
        x = (selectedText.length) * popupX;
    }

    translateIcon.css("display", "none");
    speakIconContainer.css("display", "none");

    if (showTranslateIcon){
        translateIcon.css("display", "block");
    }
    if (showSpeakIcon){
        browser.storage.local.get(['langSrcSpeakAvailable'], function(config) {
            if (config.langSrcSpeakAvailable)
                speakIconContainer.css("display", "block");
        });
    }

    buttonsContainer.css({
        left: left + x,
        top: top + OFFSET,
    });
    buttonsContainer.show();

    analyticsTrackEvent('Square Buttons', 'Show');
    browser.runtime.sendMessage({"lingvanexOpened": true});
}

function hideSquareButtons(){
    const buttonsContainer = $(`#lingvanex-container`);
    if (buttonsContainer.is(":visible")) {
        buttonsContainer.hide();

        analyticsTrackEvent('Square Buttons', 'Hide')
    }
}

function setPopupPosition(popup, x, y) {
    const PADDING = 50;

    let left = x + popupX;
    let top = y + popupY;

    popup.css({
        left: left,
        top: top
    });

    let bounding = popup[0].getBoundingClientRect();

    if (left < 0)
        left = PADDING;

    if (top < 0)
        top = PADDING;

    if (bounding.right > (window.innerWidth || document.documentElement.clientWidth))
        left = (window.innerWidth || document.documentElement.clientWidth) - bounding.width - PADDING;

    if (bounding.bottom > (window.innerHeight || document.documentElement.clientHeight))
        top = top - (bounding.bottom - (window.innerHeight || document.documentElement.clientHeight) + PADDING);

    popup.css({
        left: left,
        top: top
    });
}

function showSmallPopup(left, top, selectedText) {
    analyticsTrackEvent('Small Popup', 'Show');


    let popup = $(`#${guidContent}small-translator-div`);
    // popup.find(`.${guidContent}translatedText`).attr('data-text', selectedText);
    popup.show();
    setPopupPosition(popup, left, top);

    oneWordTranslate(selectedText, "small");
    browser.runtime.sendMessage({"lingvanexOpened": true}); // FIXME: Remove this?
}

function showMediumPopup(left, top, selectedText) {
    analyticsTrackEvent('Medium Popup', 'Show');

    let popup = $(`#${guidContent}medium-translator-div`);
    popup.show();
    setPopupPosition(popup, left, top);

    oneWordTranslate(selectedText, "medium")
}

function showBigPopup(inputText = '', outputText= '', tabName='translate') {
    let popup = $(`#${guidContent}translator-div-container`);

    popup.find(`#${guidContent}inputText`).val(inputText);
    popup.find(`#${guidTranslate}outputText`).val(outputText);

    saveInputText();
    updateExtensionData();
    showTabByName(tabName);

    popup.show();
    analyticsTrackEvent('Big Popup', 'Show');
}

function isOneWord(text) {
    let retval = false;
    if (text) {
        let arr = text.split(' ');
        arr = arr.filter(function (n) {
            return n.trim();
        });
        retval = arr.length < 2;
    }
    return retval;
}

function isMediumSentence(text) {
    let retval = false;
    if (text) {
        let arr = text.split(' ');
        arr = arr.filter(function (n) {
            return n.trim();
        });
        retval = arr.length < mediumTranslatorLimit;
    }
    return retval;
}


const onSpeakIconClick1 = event => {
    browser.storage.local.get(['langSrc'], function(config) {
        analyticsTrackEvent('Square Buttons', 'Click Speak', {'lang_src': config.langSrc, 'text': selectedText});
        speakText(selectedText, config.langSrc, $(event.target).parent());
    })
};

function onTranslationIconClick(e) {
    debug_log('onTranslationIconClick');

    analyticsTrackEvent('Square Buttons', 'Click Translate', {'text': selectedText});
    e.stopPropagation(); // Disable document onMouseUp handler

    hideSquareButtons();

    browser.storage.local.get(['popupX', 'popupY'], function ({popupX, popupY}) {
        if (popupX !== undefined) {
            $(`#${guidContent}translator-div-container`).css({
                top: popupY + "px",
                left: popupX + "px"
            })
        }
    });


    if (isOneWord(selectedText)) {
        updateExtensionData();
        showSmallPopup(e.pageX, e.pageY - 25, selectedText);

        // oneWordTranslate(selectedText, "small");
        // $(`.${guidContent}small-translator-div`).show()

    } else if (isMediumSentence(selectedText)) {
        updateExtensionData();
        showMediumPopup(e.pageX, e.pageY - 25, selectedText);
        //oneWordTranslate(selectedText, "medium");
        //translate();
        //$(`#${guidContent}medium-translator-div`).show();
        //getLanguagesForAnalytics('Phrase screen', 'Shown')
    } else {
        // hideSmallPopup();
        // hideMediumPopup();
        updateExtensionData();
        clearPopups();

        showBigPopup(selectedText);
        translate();
    }
}


const makePopupDraggable = () => {
    $(`#${guidContent}translator-div-container`).draggable({
        stop: function () {
            let offset = this.getBoundingClientRect(); // $(this).offset();

            let xPos = offset.left % ((window.innerWidth || document.documentElement.clientWidth) - 10);
            let yPos = offset.top % ((window.innerHeight || document.documentElement.clientHeight) - 10);

            browser.storage.local.set({"popupY": yPos, "popupX": xPos}, null);
        }
    });
};

function getSelectedText() {
    let text = '';
    if (window.getSelection) {
        let selection = window.getSelection();
        if (selection.anchorNode && selection.anchorNode.nodeType === Node.TEXT_NODE) {
            text = window.getSelection().toString();
        }
    }
    else if (document.getSelection) {
        let selection = document.getSelection();
        if (selection.anchorNode && selection.anchorNode.nodeType === Node.TEXT_NODE) {
            text = document.getSelection().toString();
        }
    }
    else if (document.selection) {
        text = document.selection.createRange().text;
    }
    else
        return;

    return text.trim();
}

function receiveMessage(event) {
    if (event.data == "lingvanexCloseFrame") {
        //$(`#${guidContent}translator-div-container`).css({display: 'none'});
        $(`#${guidContent}translator-div-container`).hide();
    }
}

window.addEventListener("message", receiveMessage, false);

// Inform the background page that
// this tab should have a page-action.
browser.runtime.sendMessage({
    from: 'content',
    subject: 'getSelection',
});

browser.extension.onMessage.addListener(function (request, sender, callback) {
    if (request.msg === 'showBigPopup') {
        hideSquareButtons();
        hideSmallPopup();
        hideMediumPopup();
        showBigPopup(request.inputText, request.outputText, request.tabName);
        translate();
    }
});

// Listen for messages from the popup.
browser.runtime.onMessage.addListener((request, sender, response) => {
    // First, validate the message's structure.
    if ((request.from === 'popup') && (request.subject === 'getSelection')) {
        // Collect the necessary data.
        // (For your specific requirements `document.querySelectorAll(...)`
        //  should be equivalent to jquery's `$(...)`.)

        // Directly respond to the sender (popup),
        // through the specified callback.
        console.debug('getSelection');
        response(getSelectedText());
    }
});
